use crate::git::error::SocialError;
use crate::git::repo::SocialRepo;
use crate::git::{SocialBlame, SocialBlameHunk};
use git2::{Blame, BlameHunk, BlameOptions};
use std::io::{BufRead, BufReader};

fn into_social_blame_hunk(bh: BlameHunk, line: String) -> SocialBlameHunk {
    SocialBlameHunk {
        signature: bh.final_signature().into(),
        commit_id: format!("{}", bh.orig_commit_id()),
        line,
    }
}

fn into_social_blame(file: String, blame: Blame<'_>, buffer: BufReader<&[u8]>) -> SocialBlame {
    let mut hunks: Vec<SocialBlameHunk> = Vec::new();
    for (i, line) in buffer.lines().enumerate() {
        if let (Ok(line), Some(hunk)) = (line, blame.get_line(i + 1)) {
            let shunk = into_social_blame_hunk(hunk, line);
            hunks.push(shunk);
        }
    }
    SocialBlame { file, hunks }
}

pub trait GitBlame {
    fn blame(&self, from_oid: &str, path: &str) -> Result<SocialBlame, SocialError>;
}

impl GitBlame for SocialRepo {
    fn blame(&self, from_oid: &str, path: &str) -> Result<SocialBlame, SocialError> {
        let revspec = self.repo.revparse_single(from_oid)?;
        let commit_id = revspec.id();

        let spec = format!("{}:{}", commit_id, path);
        let object = self.repo.revparse_single(&spec[..])?;
        let blob = self.repo.find_blob(object.id())?;
        let reader = BufReader::new(blob.content());

        let mut opts = BlameOptions::new();
        opts.track_copies_any_commit_copies(true)
            .track_copies_same_commit_copies(true)
            .newest_commit(revspec.id());

        self.repo
            .blame_file(std::path::Path::new(path), Some(&mut opts))
            .map(|blame| into_social_blame(path.to_owned(), blame, reader))
            .map_err(|d| d.into())
    }
}
