use crate::git::error::SocialError;
use crate::git::repo::SocialRepo;
use crate::git::SocialDiff;
use git2::{Diff, DiffOptions};
use git2::{DiffFormat, DiffLine};

fn into_social_diff(d: Diff<'_>) -> SocialDiff {
    let mut lines: Vec<String> = Vec::new();
    d.print(DiffFormat::Patch, |_, _, line: DiffLine| {
        if let Ok(l) = String::from_utf8(line.content().to_vec()) {
            lines.push(format!("{} {}", line.origin(), l));
        } else {
            lines.push("? cannot convert utf8 string on this line!".to_string());
        }
        true
    })
    .unwrap_or(());
    SocialDiff { lines }
}

pub trait Show {
    fn show(&self, from_oid: &str, to_oid: &str) -> Result<SocialDiff, SocialError>;
}

impl Show for SocialRepo {
    fn show(&self, from_oid: &str, to_oid: &str) -> Result<SocialDiff, SocialError> {
        let mut opts = DiffOptions::new();
        opts.reverse(false)
            .force_text(false)
            .ignore_whitespace_eol(false)
            .ignore_whitespace_change(false)
            .ignore_whitespace(false)
            .include_ignored(false)
            .include_untracked(false)
            .patience(true)
            .context_lines(5)
            .minimal(false);

        let t1 = self.get_tree(from_oid).ok();
        let t2 = self.get_tree(to_oid).ok();

        self.repo
            .diff_tree_to_tree(t1.as_ref(), t2.as_ref(), Some(&mut opts))
            .map(into_social_diff)
            .map_err(|d| d.into())
    }
}
