pub(crate) mod blame;
pub(crate) mod diff;
pub(crate) mod error;
pub(crate) mod log;
pub(crate) mod references;
pub(crate) mod repo;
pub(crate) mod tree;

use chrono::offset::{Local, TimeZone};
use chrono::{DateTime, NaiveDateTime};
use git2::{Commit, Signature};

fn min_date() -> DateTime<Local> {
    Local.from_utc_datetime(&NaiveDateTime::MIN)
}

fn timestamp_with_fallback(timestamp: i64) -> DateTime<Local> {
    Local.timestamp_opt(timestamp, 0).earliest().unwrap_or(min_date())
}

#[derive(Clone, Debug)]
pub struct SocialSignature {
    pub when: DateTime<Local>,
    pub name: String,
    pub email: String,
}

impl Default for SocialSignature {
    fn default() -> Self {
        Self {
            when: min_date(),
            name: "".to_string(),
            email: "".to_string(),
        }
    }
}

#[derive(Clone, Debug)]
pub struct SocialCommit {
    pub id: String,
    pub time: DateTime<Local>,
    pub message: String,
    pub author: SocialSignature,
    pub committer: SocialSignature,
    pub diff: Option<SocialDiff>,
    pub parent_ids: Option<Vec<String>>, // TODO: Use iterator instead of vec.
}

impl SocialCommit {
    pub fn oneline(self) -> Self {
        Self {
            message: self.message.lines().next().unwrap_or("").to_owned(),
            parent_ids: None,
            diff: None,
            ..self
        }
    }
}

impl Default for SocialCommit {
    fn default() -> Self {
        Self {
            id: "".to_string(),
            time: min_date(),
            message: "".to_string(),
            author: SocialSignature::default(),
            committer: SocialSignature::default(),
            diff: None,
            parent_ids: None,
        }
    }
}

impl From<Signature<'_>> for SocialSignature {
    fn from(s: Signature<'_>) -> Self {
        Self {
            name: s.name().unwrap_or("").to_owned(),
            email: s.email().unwrap_or("").to_owned(),
            when: timestamp_with_fallback(s.when().seconds()),
        }
    }
}

impl From<Commit<'_>> for SocialCommit {
    fn from(c: Commit<'_>) -> Self {
        Self {
            id: format!("{}", c.id()),
            time: timestamp_with_fallback(c.time().seconds()),
            message: c.message().unwrap_or("").to_owned(),
            author: c.author().into(),
            committer: c.committer().into(),
            diff: None,
            parent_ids: None,
        }
    }
}

#[derive(Clone, Debug, Default)]
pub struct SocialDiff {
    pub lines: Vec<String>,
}

#[derive(Clone, Debug, Default)]
pub struct SocialTreeEntry {
    pub id: String,
    pub name: String,
    pub kind: String,
    pub filemode: i32,
    pub size: usize,
}

#[derive(Clone, Debug, Default)]
pub struct SocialFileContent {
    pub path: String,
    pub content: String,
}

#[derive(Clone, Debug, Default)]
pub struct SocialBranch {
    pub name: String,
    pub head: SocialCommit,
}

#[derive(Clone, Debug, Default)]
pub struct SocialTag {
    pub name: String,
    pub head: SocialCommit,
    pub annotation: Option<String>,
}

#[derive(Clone, Debug, Default)]
pub struct SocialRefs {
    pub branches: Vec<SocialBranch>,
    pub tags: Vec<SocialTag>,
}

#[derive(Clone, Debug, Default)]
pub struct SocialBlameHunk {
    pub commit_id: String,
    pub line: String,
    pub signature: SocialSignature,
}

#[derive(Clone, Debug, Default)]
pub struct SocialBlame {
    pub file: String,
    pub hunks: Vec<SocialBlameHunk>,
}

#[derive(Clone, Debug, Default)]
pub struct SocialRepoInfo {
    pub name: String,
    pub author: SocialSignature,
    pub description: String,
}
