use crate::git::log::into_social_commit;
use crate::git::repository::SocialRepo;
use crate::git::Error;
use git2::{Branch, BranchType, Commit};
use shared::{SocialBranch, SocialCommit, SocialTag};

fn into_social_branch(b: Branch<'_>) -> SocialBranch {
    SocialBranch {
        name: b.name().unwrap_or(Some("")).unwrap_or("").to_owned(),
        head: SocialCommit::default(),
    }
}

fn into_social_tag(name: Option<&str>) -> SocialTag {
    SocialTag {
        name: name.unwrap_or("").to_owned(),
        head: SocialCommit::default(),
        annotation: None,
    }
}

pub trait Refs {
    fn branches(&self) -> Result<Vec<SocialBranch>, Error>;
    fn tags(&self) -> Result<Vec<SocialTag>, Error>;
}

impl Refs for SocialRepo {
    fn branches(&self) -> Result<Vec<SocialBranch>, Error> {
        let all_branches = self.repo.branches(Some(BranchType::Local))?;
        let mut resulting_brances: Vec<SocialBranch> = vec![];

        for r in all_branches {
            r.map(|(branch, _)| {
                let mut sb = into_social_branch(branch);
                self.repo
                    .revparse_single(&sb.name)
                    .map(|obj| {
                        obj.peel_to_commit()
                            .map(|c: Commit| {
                                sb.head = into_social_commit(c);
                                resulting_brances.push(sb);
                            })
                            .unwrap_or(());
                    })
                    .unwrap_or(());
            })
            .unwrap_or(());
        }
        Ok(resulting_brances)
    }

    fn tags(&self) -> Result<Vec<SocialTag>, Error> {
        let tagnames = self.repo.tag_names(None)?;
        let mut result: Vec<SocialTag> = vec![];

        for t in tagnames.into_iter() {
            let mut st = into_social_tag(t);
            let _ = self.repo.revparse_single(&st.name).map(|obj| {
                obj.peel_to_commit().map(|c: Commit| {
                    st.head = into_social_commit(c);
                    result.push(st);
                })
            });
        }
        Ok(result)
    }
}
