use crate::git::error::SocialError;
use crate::git::repo::SocialRepo;
use crate::git::{SocialFileContent, SocialTreeEntry};
use git2::{Blob, Error as GitError, Object, ObjectType, TreeEntry};
use std::path::Path;

fn into_social_tree_entry(te: TreeEntry<'_>, size: usize) -> SocialTreeEntry {
    SocialTreeEntry {
        id: format!("{}", te.id()),
        name: te.name().unwrap_or("").to_owned(),
        kind: te.kind().map(|ot| ot.str()).unwrap_or("any").to_owned(),
        filemode: te.filemode(),
        size,
    }
}

pub trait LsTree {
    fn ls_tree(&self, rev: &str, path: &Path) -> Result<Vec<SocialTreeEntry>, SocialError>;
    fn show_file(&self, rev: &str, path: &Path) -> Result<SocialFileContent, SocialError>;
}

impl LsTree for SocialRepo {
    fn ls_tree(&self, rev: &str, path: &Path) -> Result<Vec<SocialTreeEntry>, SocialError> {
        let matched_tree = if path.to_str().unwrap_or("") == "" {
            self.get_tree(rev)?
        } else {
            self.get_tree(rev)
                .and_then(|t| t.get_path(path))
                .and_then(|entry: TreeEntry| self.repo.find_tree(entry.id()))?
        };

        let mut result: Vec<SocialTreeEntry> = vec![];
        let mut files: Vec<SocialTreeEntry> = vec![];
        for entry in matched_tree.iter() {
            match entry.kind() {
                Some(git2::ObjectType::Blob) => {
                    self.repo
                        .revparse_single(&format!("{}", entry.id()))
                        .map(|obj: Object| {
                            if let Some(b) = obj.as_blob() {
                                files.push(into_social_tree_entry(entry, b.content().len()))
                            };
                        })
                        .unwrap_or(());
                }
                Some(git2::ObjectType::Tree) => result.push(into_social_tree_entry(entry, 0)),
                _ => (),
            }
        }
        result.extend(files);
        Ok(result)
    }

    fn show_file(&self, rev: &str, path: &Path) -> Result<SocialFileContent, SocialError> {
        self.get_tree(rev)
            .and_then(|t| t.get_path(path))
            .and_then(|entry: TreeEntry| match entry.kind() {
                Some(ObjectType::Blob) => self
                    .repo
                    .revparse_single(&format!("{}", entry.id()))
                    .and_then(|obj: Object| {
                        obj.as_blob()
                            .ok_or_else(|| {
                                GitError::from_str(&format!("Cannot get blob for {}", obj.id()))
                            })
                            .and_then(|blob: &Blob| {
                                String::from_utf8(blob.content().to_vec()).map_err(|_| {
                                    GitError::from_str(format!("Cannot read file {}", path.display()).as_ref())
                                })
                            })
                            .map(|content| SocialFileContent {
                                content,
                                path: format!("{}", path.display()),
                            })
                    }),
                _ => Result::Err(GitError::from_str("Object is not a Blob!")),
            })
            .map_err(|e| e.into())
    }
}
